# set cdash buildname
set(BUILDNAME
    "${CMAKE_SYSTEM_NAME}-${CMAKE_HOST_SYSTEM_PROCESSOR}-${CMAKE_Fortran_COMPILER_ID}-${BLAS_TYPE}-${cmake_build_type_tolower}"
    CACHE STRING
    "Name of build on the dashboard"
    )

# set ctest own timeout
set(DART_TESTING_TIMEOUT
    "1500"
    CACHE STRING
    "Set timeout in seconds for every single test"
    )
message(STATUS "Set CDash default timeout for single test set to ${DART_TESTING_TIMEOUT} seconds. Overwritten by test's TIMEOUT property label and, ultimatively, by pam timeout setting.")

macro(dirac_test _name _label _timeout)
    add_test(${_name} ${PYTHON_EXECUTABLE} ${PROJECT_SOURCE_DIR}/test/${_name}/test --binary-dir=${PROJECT_BINARY_DIR} --work-dir=${PROJECT_BINARY_DIR}/test/${_name} --verbose)
    if(NOT "${_label}" STREQUAL "")
        #miro: test's TIMEOUT marker goes after the LABEL marker
        if(NOT "${_timeout}" STREQUAL "")
            set_tests_properties(${_name} PROPERTIES LABELS "${_label}" TIMEOUT "${_timeout}")
            message(STATUS "Test's '${_name}' timeout set to ${_timeout} seconds. Can be overwritten by pam's timeout setting.")
        else()
            set_tests_properties(${_name} PROPERTIES LABELS "${_label}")
        endif()
    endif()
endmacro()

# activate ctest
include(CTest)
enable_testing()
# these tests apeear are also in release-xx versions
dirac_test(dft_alda_xcfun "long;dft" "")
dirac_test(dft_alda "long;dft" "")
dirac_test(stex "long" "")
dirac_test(geo_opt "long" "")
dirac_test(geo_opt_xyz "long" "")
dirac_test(geo_opt_symm_xyz "long" "")
#dirac_test(london_properties_symmetry "long;lao" "")
dirac_test(response_pv_nmr "long" "")
dirac_test(krci_energy "long" "")
dirac_test(molecular_mean_field_hamiltonian_schemes "long" "")
dirac_test(electrostatic_potential "long" "")
dirac_test(mp2_properties "long;mp2" "")
dirac_test(ffpt_dipmom_polariz_relcc "cc" "")
dirac_test(reladc_sip "reladc" "")
dirac_test(reladc_sipeigv "reladc" "")
dirac_test(reladc_dip "reladc" "")
dirac_test(reladc_fano "reladc" "")
dirac_test(polprp_ph "polprp" "")
dirac_test(cc_energy_and_mp2_dipole "cc" "")
dirac_test(eomcc "cc" "")
dirac_test(eomip_energy_symmetry "cc" "")
dirac_test(eomea_energy_symmetry "cc" "")
dirac_test(eomee_energy_symmetry "cc" "")
dirac_test(count_cc_memory "cc" "")
dirac_test(cisd_energy "ci" "")
dirac_test(cosci_average "ci" "")
dirac_test(dft_ao_eval "dft" "")
dirac_test(dft_geo_opt "" "")
dirac_test(dft_response "response;dft" "")
dirac_test(dft_response_symmetry "response;dft" "")
dirac_test(dft_response_xcfun "response;dft" "")
dirac_test(efg "efg" "")
dirac_test(fde_static-vemb_dipole "fde" "")
dirac_test(fde_response_autodiff "fde" "")
dirac_test(fde_export_density "fde" "")
dirac_test(fde_interaction_energy "fde" "")
dirac_test(fde_response_mag "fde;magnetizabilities;lao" "")
dirac_test(fde_response_shield "fde;shielding;lao" "")
dirac_test(frozen_orbitals "" "")
dirac_test(fscc_restart "restart" "")
dirac_test(cc_restart "restart" "")
dirac_test(fscc "fscc" "")
dirac_test(fscc_highspin "fscc" "")
dirac_test(ihfscc_newshift "fscc" "")
dirac_test(geo_opt_saddle "" "")
dirac_test(huckel_start "" "")
dirac_test(lucita_large "krci" "")
dirac_test(mcscf_energy "mcscf" "")
dirac_test(response_excitation_energies "" "")
dirac_test(response_nmr_spin-spin "response;nmr" "")
dirac_test(response_qr "response" "")
dirac_test(visual_lao_current "visual;lao" "")
dirac_test(visual_lao_symmetry "visual;lao" "")
dirac_test(xyz_input "input" "")
dirac_test(krci_energy_q_corrections "medium;krci;qcorr;ci" "")
dirac_test(dirrci_property "ci" "")
dirac_test(general_property_krci "krci" "")
dirac_test(acmoin "short" "")
dirac_test(atomic_start "short" "")
dirac_test(basis_automatic_augmentation "basis;short" "")
dirac_test(basis_contraction "basis;short" "")
dirac_test(basis_input "basis;short" "")
dirac_test(basis_input_scripted "basis;long" "3600")
dirac_test(bsse "basis;short" "")
dirac_test(bss_energy "short" "")
dirac_test(cc_essential "short;cc" "")
dirac_test(cc_linear "short;cc" "")
dirac_test(cc_gradient "cc_gradient" "")
dirac_test(cosci_energy_spinfree "short;ci" "")
dirac_test(cosci_methods "short;ci" "")
dirac_test(cosci_tmom "short;ci" "")
dirac_test(density_at_nuclei "short" "")
dirac_test(dft_betasigma "short;dft" "")
dirac_test(dft_cam "short;dft" "")
dirac_test(dft_erf_xcfun "short;dft" "")
dirac_test(dft_lao_mag_prop "short;dft;lao" "")
dirac_test(dft_pp86 "short;dft" "")
dirac_test(ecp "short" "")
dirac_test(free-particle_projection "short" "")
dirac_test(gaunt "short;gaunt" "")
dirac_test(krci_properties_perm_dipmom "short;krci" "")
dirac_test(linear_structures "short" "")
dirac_test(mcscf_properties "short;mcscf" "")
dirac_test(mp2_energy "short;mp2" "")
dirac_test(mp2_srdft_energies "short;mp2;srdft" "")
dirac_test(mp2_natural_orbitals "short;mp2" "")
dirac_test(open-shells "short" "")
dirac_test(projection_analysis "short;projection" "")
dirac_test(pvc_scf "short;pvc;scf" "")
dirac_test(response_C6 "short;response" "")
dirac_test(response_hf_polarizability "short;response;polarizabilities" "")
dirac_test(response_lao_shield "short;response;shielding;lao" "")
dirac_test(response_lao_magnetiz "short;response;magnetizabilities;lao" "")
dirac_test(response_lao_magnetiz_dft "short;response;magnetizabilities;lao;dft" "")
dirac_test(response_nmr_levy-leblond "short;response;nmr" "")
dirac_test(response_rkbimp_shield "short;response" "")
dirac_test(symmetry_recognition "short" "")
dirac_test(visual_div_rot "short;visual" "")
dirac_test(visual_lao_shielding "short;visual;lao" "")
dirac_test(visual_frac_occupation "short;visual" "")
dirac_test(lucita_short "short;krci" "")
dirac_test(xyz_symmetry_recognition "short" "")
dirac_test(zora "short;zora" "")
dirac_test(dft_ac "short;dft" "")
dirac_test(dft_grid_export_import "short;dft" "")
dirac_test(localization "short" "")
dirac_test(krci_properties_omega_tdm "short;krci" "")
dirac_test(visual_elf "short;visual" "")
dirac_test(visual_gamma5 "short;visual" "")
dirac_test(visual_general "short;visual" "")
dirac_test(symmetry_scf_energy "short;scf" "")
dirac_test(scf_levelshift "short;scf" "")
dirac_test(response_complex "short;response" "")
dirac_test(response_nonrel "short;response" "")
dirac_test(cosci_energy "short;ci" "")
dirac_test(lucita_q_corrections "short;qcorr;ci" "")
dirac_test(x2c-SCF_to_4c-SCF "short;x2c;4c;scf" "")
dirac_test(import_mos "short;import;scf;mcscf" "")
dirac_test(operators_mo_mtx_elements "short;operators" "")
# disabing eomcc expectation value test until the code is verified
#dirac_test(eomcc_expval "short;cc" "")

